// create spec
var spec = {
    $schema: "https://vega.github.io/schema/vega/v5.json",
    description: "An interactive world map supporting pan and zoom.",
    width: 1200,
    height: 500,
    autosize: "none",
    signals: [
      { name: "tx", "update": "width / 2" },
      { name: "ty", "update": "height / 2" },
      {
        name: "scale",
        value: 150,
        on: [{
          events: { type: "wheel", consume: true },
          update: "clamp(scale * pow(1.0005, -event.deltaY * pow(16, event.deltaMode)), 150, 3000)"
        }]
      },
      {
        name: "angles",
        value: [0, 0],
        on: [{
          events: "mousedown",
          update: "[rotateX, centerY]"
        }]
      },
      {
        name: "cloned",
        value: null,
        on: [{
          events: "mousedown",
          update: "copy('projection')"
        }]
      },
      {
        name: "start",
        value: null,
        on: [{
          events: "mousedown",
          update: "invert(cloned, xy())"
        }]
      },
      {
        name: "drag", "value": null,
        on: [{
          events: "[mousedown, window:mouseup] > window:mousemove",
          update: "invert(cloned, xy())"
        }]
      },
      {
        name: "delta", "value": null,
        on: [{
          events: { signal: "drag"},
          update: "[drag[0] - start[0], start[1] - drag[1]]"
        }]
      },
      {
        name: "rotateX", "value": 0,
        on: [{
          events: { signal: "delta"},
          update: "angles[0] + delta[0]"
        }]
      },
      {
        name: "centerY", "value": 0,
        on: [{
          events: { signal: "delta"},
          update: "clamp(angles[1] + delta[1], -60, 60)"
        }]
      }
    ],
  
    projections: [
      {
        name: "projection",
        type: "mercator",
        scale: { signal: "scale"},
        rotate: [{ signal: "rotateX"}, 0, 0],
        center: [0, { signal: "centerY"}],
        translate: [
            { signal: "tx"}, 
            { signal: "ty"}]
      }
    ],
    data: [
      {
        name: "gapminder",
        url: "https://raw.githubusercontent.com/picoral/csc-444-data/main/life_expectancy_2020.csv",
        format: { type: "csv" }
      },
      {
        name: "world",
        url: "https://raw.githubusercontent.com/vega/datalib/master/test/data/world-110m.json",
        format: {
          type: "topojson",
          feature: "countries",
        },
        transform: [
          { 
            type: "lookup",
            from: "gapminder",
            key: "id",
            fields: ["id"],
            values: ["lifeExpectancy", "country"] }
        ]
      },
      {
        name: "graticule",
        transform: [
          { type: "graticule", step: [15, 15] }
        ]
      }
    ],
    scales: [
        {
            name: "fillScale",
            type: "linear",
            domain: { data: "gapminder", field: "lifeExpectancy" },
            range: { scheme: "oranges" }
        }
    ],
    marks: [
      {
        type: "shape",
        from: { data: "graticule"},
        encode: {
          enter: {
            strokeWidth: { value: 1},
            stroke: { value: "#ddd"},
            fill: { value: null}
          }
        },
        transform: [
          { type: "geoshape", projection: "projection" }
        ]
      },
      {
        type: "shape",
        from: { data: "world"},
        encode: {
          enter: {
            fill: { field: "lifeExpectancy", scale: "fillScale" },
            tooltip: { signal: "datum.country + ' ' + datum.lifeExpectancy"}
          },
          hover: {
            stroke: { value: "red" },
            strokeWidth: { value: 1.5}
          },
          update: {
            stroke: { value: "#bbb"},
            strokeWidth: { value: 0.5}
          }
        },
        transform: [
          { type: "geoshape", projection: "projection" }
        ]
      }
    ],
    legends: [
      {
        fill: "fillScale",
        orient: "bottom-right",
        title: "Life Expectancy"
      }
    ]
  };

// create runtime
var runtime = vega.parse(spec);

// create view
var view = new vega.View(runtime)
                   .logLevel(vega.Error)
                   .renderer("svg")
                   .initialize("#view")
                   .hover();

// run it
view.run();